#[repr(transparent)]pub struct FastCodePointTrie<'trie, T: TrieValue> {
inner: CodePointTrie<'trie, T>,
}Expand description
Type-safe wrapper for a fast trie guaranteeing
the the getters don’t branch on the trie type
and for guarenteeing that get16 is branchless
in release builds.
Fields
inner: CodePointTrie<'trie, T>Trait Implementations
Sourceimpl<'trie, T: TrieValue> AbstractCodePointTrie<'trie, T> for FastCodePointTrie<'trie, T>
impl<'trie, T: TrieValue> AbstractCodePointTrie<'trie, T> for FastCodePointTrie<'trie, T>
Sourceunsafe fn utf8_two_byte(&self, high_five: u32, low_six: u32) -> T
unsafe fn utf8_two_byte(&self, high_five: u32, low_six: u32) -> T
Sourceunsafe fn utf8_three_byte(&self, high_ten: u32, low_six: u32) -> T
unsafe fn utf8_three_byte(&self, high_ten: u32, low_six: u32) -> T
Sourcefn bmp(&self, bmp: u16) -> T
fn bmp(&self, bmp: u16) -> T
Sourcefn supplementary(&self, supplementary: u32) -> T
fn supplementary(&self, supplementary: u32) -> T
Sourcefn code_point(&self, code_point: u32) -> T
fn code_point(&self, code_point: u32) -> T
Sourceimpl<'trie, T: Clone + TrieValue> Clone for FastCodePointTrie<'trie, T>
impl<'trie, T: Clone + TrieValue> Clone for FastCodePointTrie<'trie, T>
Sourcefn clone(&self) -> FastCodePointTrie<'trie, T>
fn clone(&self) -> FastCodePointTrie<'trie, T>
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<'trie, T: PartialEq + TrieValue> PartialEq for FastCodePointTrie<'trie, T>
impl<'trie, T: PartialEq + TrieValue> PartialEq for FastCodePointTrie<'trie, T>
Sourcefn eq(&self, other: &FastCodePointTrie<'trie, T>) -> bool
fn eq(&self, other: &FastCodePointTrie<'trie, T>) -> bool
self and other values to be equal, and is used by ==.Sourceimpl<'trie, T: TrieValue> TryFrom<&'trie CodePointTrie<'trie, T>> for &'trie FastCodePointTrie<'trie, T>
impl<'trie, T: TrieValue> TryFrom<&'trie CodePointTrie<'trie, T>> for &'trie FastCodePointTrie<'trie, T>
Sourcetype Error = TypedCodePointTrieError
type Error = TypedCodePointTrieError
Sourcefn try_from(
reference: &'trie CodePointTrie<'trie, T>,
) -> Result<&'trie FastCodePointTrie<'trie, T>, TypedCodePointTrieError>
fn try_from( reference: &'trie CodePointTrie<'trie, T>, ) -> Result<&'trie FastCodePointTrie<'trie, T>, TypedCodePointTrieError>
Sourceimpl<'trie, T: TrieValue> TryFrom<CodePointTrie<'trie, T>> for FastCodePointTrie<'trie, T>
impl<'trie, T: TrieValue> TryFrom<CodePointTrie<'trie, T>> for FastCodePointTrie<'trie, T>
Sourcetype Error = TypedCodePointTrieError
type Error = TypedCodePointTrieError
Sourcefn try_from(
value: CodePointTrie<'trie, T>,
) -> Result<FastCodePointTrie<'trie, T>, TypedCodePointTrieError>
fn try_from( value: CodePointTrie<'trie, T>, ) -> Result<FastCodePointTrie<'trie, T>, TypedCodePointTrieError>
Sourceimpl<'trie, T: TrieValue> TypedCodePointTrie<'trie, T> for FastCodePointTrie<'trie, T>
impl<'trie, T: TrieValue> TypedCodePointTrie<'trie, T> for FastCodePointTrie<'trie, T>
Sourcefn as_untyped_ref(&self) -> &CodePointTrie<'trie, T>
fn as_untyped_ref(&self) -> &CodePointTrie<'trie, T>
Returns a reference to the wrapped CodePointTrie.
Sourcefn to_untyped(self) -> CodePointTrie<'trie, T>
fn to_untyped(self) -> CodePointTrie<'trie, T>
Extracts the wrapped CodePointTrie.
Sourcefn get16(&self, bmp: u16) -> T
fn get16(&self, bmp: u16) -> T
Lookup trie value by Basic Multilingual Plane Code Point without branching on trie type.
Sourceunsafe fn get_utf8_three_byte(&self, high_ten: u32, low_six: u32) -> T
unsafe fn get_utf8_three_byte(&self, high_ten: u32, low_six: u32) -> T
Returns the value that is associated with a three-byte UTF-8 or WTF-8 sequence.
high_ten is the low four bits of the lead byte of three-byte UTF-8 or WTF-8 sequence shifted left by 6 followed by the low six bits of the first trail byte.
low_six is the low six bits of the last trail byte of a three-byte UTF-8 or WTF-8 sequence.
Sequences representing surrogates (WTF-8) are allowed.
Safety
high_ten must not have bit positions other than the lowest 10 set to 1.
low_six must not have bit positions other than the lowest 6 set to 1.
Panics
With debug assertions enabled, panics if the above safety invariants are
violated or high_ten is out of range for three-byte WTF-8 (or UTF-8)
sequence.
Sourcefn get32_u32(&self, code_point: u32) -> u32
fn get32_u32(&self, code_point: u32) -> u32
u32 by Unicode Scalar Value without branching on trie type.Sourcefn get8(&self, latin1: u8) -> T
fn get8(&self, latin1: u8) -> T
Sourceunsafe fn get7(&self, ascii: u8) -> T
unsafe fn get7(&self, ascii: u8) -> T
Sourcefn get32_supplementary(&self, supplementary: u32) -> T
fn get32_supplementary(&self, supplementary: u32) -> T
Sourcefn get(&self, c: char) -> T
fn get(&self, c: char) -> T
Sourcefn get32(&self, code_point: u32) -> T
fn get32(&self, code_point: u32) -> T
Sourcefn get32_by_fast_index(&self, code_point: u32) -> Option<T>
fn get32_by_fast_index(&self, code_point: u32) -> Option<T>
code_point in this CodePointTrie
if code_point uses fast-path lookup or None if code_point
should use small-path lookup or is above the supported range.Sourceimpl<'yoke, T> Yokeable<'yoke> for FastCodePointTrie<'static, T>where
T: 'static + TrieValue,
impl<'yoke, T> Yokeable<'yoke> for FastCodePointTrie<'static, T>where
T: 'static + TrieValue,
Sourcetype Output = FastCodePointTrie<'yoke, T>
type Output = FastCodePointTrie<'yoke, T>
Self with the 'static replaced with 'a, i.e. Self<'a>Sourcefn transform_owned(self) -> Self::Output
fn transform_owned(self) -> Self::Output
Sourceimpl<'zf, 'zf_inner, T: TrieValue> ZeroFrom<'zf, FastCodePointTrie<'zf_inner, T>> for FastCodePointTrie<'zf, T>
impl<'zf, 'zf_inner, T: TrieValue> ZeroFrom<'zf, FastCodePointTrie<'zf_inner, T>> for FastCodePointTrie<'zf, T>
Sourcefn zero_from(this: &'zf FastCodePointTrie<'zf_inner, T>) -> Self
fn zero_from(this: &'zf FastCodePointTrie<'zf_inner, T>) -> Self
C into a struct that may retain references into C.